const { Client } = require('@elastic/elasticsearch');
const casual = require('casual');
require('array.prototype.flatmap').shim();

const elasticSearchClient = new Client({
  node: "http://localhost:9200"
});



async function createIndexIfNotExists(indexName) {
  let result = await elasticSearchClient.indices.exists({
    index: indexName
  });
  if (result.body === true) {
    console.log(indexName + ' already exists');
  } else {
    console.log(indexName + ' will be created');
    await createIndex(indexName);
  }
}

async function createIndex(indexName) {
  let result = await elasticSearchClient.indices.create({
    index: indexName,
    body: {
      "mappings": {
        "properties": {
          "companyName": {
            "type": "text"
          },
          "customerId": {
            "type": "integer"
          },
          "dateOfBirth": {
            "type": "date"
          },
          "email": {
            "type": "text"
          },
          "firstName": {
            "type": "text",
            "copy_to": "fullName"
          },
          "middleName": {
            "type": "text",
            "copy_to": "fullName"
          },
          "lastName": {
            "type": "text",
            "copy_to": "fullName"
          },
          "fullName": {
            "type": "text"
          },
          "mobileNumber": {
            "type": "text"
          },
          "officeNumber": {
            "type": "text"
          },
          "address": {
            "properties": {
              "line1": {
                "type": "text",
                "copy_to": "fullAddress"
              },
              "line2": {
                "type": "text",
                "copy_to": "fullAddress"
              },
              "city": {
                "type": "text",
                "copy_to": "fullAddress"
              },
              "state": {
                "type": "text",
                "copy_to": "fullAddress"
              },
              "zip": {
                "type": "text",
                "copy_to": "fullAddress"
              }
            }
          },
          "fullAddress": {
            "type": "text"
          }
        }
      }
    }
  });
  console.log(result.statusCode);
}

casual.define('customer', function () {
  return {
    customerId: 0, // will update in the loop
    firstName: casual.first_name,
    middleName: casual.first_name,
    lastName: casual.last_name,
    address: {
      line1: casual.address1,
      line2: casual.address2,
      city: casual.city,
      state: casual.state,
      zip: casual.zip(digits = 9)
    },
    email: casual.email,
    dateOfBirth: casual.date(format = 'YYYY-MM-DD'),
    mobileNumber: casual.phone,
    officeNumber: casual.phone,
    companyName: casual.company_name + " " + casual.company_suffix
  }
});

function generateCustomers(count) {
  let customers = [];
  for (let index = 1; index <= count; index++) {
    let customer = casual.customer;
    customer.customerId = index;
    customers.push(customer);
  }
  return customers;
}

async function indexCustomers(customers, indexName) {
  console.log('Seeding...')
  const body = customers.flatMap(customer => [{ index: { _index: indexName, _id: customer.customerId, _type: '_doc' } }, customer]);
  const { body: bulkResponse } = await elasticSearchClient.bulk({ refresh: true, body })

}

async function seed(indexName, count) {
  await createIndexIfNotExists(indexName);
  
  let customers = generateCustomers(count);
  await indexCustomers(customers, indexName);
}

seed("customers", 5000);